###########################################
# title:_SetTargeFeatureUpdateVersion.ps1
# created by  : olympus
# created date: 2022/02/16
# last update : 2022/02/16
###########################################
$argTargetOSVersion = $args[1]
$ErrorActionPreference = "Stop"
$wsobj = new-object -comobject wscript.shell
$now = Get-Date -Format 'yyyyMMdd-HHmmss';
$logname = "SetWindowsUpdateDefer_$now.log"

# Utility method
Function Write-Log
{
	[CmdletBinding()]
	param(
		[Parameter()]
		[ValidateNotNullOrEmpty()]
		[ValidateSet("DEBUG",'INFO','WARN','ERROR')]
		[string]$Level = 'INFO',
		
		[Parameter()]
		[ValidateNotNullOrEmpty()]
		#[string]$Message
		$Message
	)
	
	$Time = (Get-Date -Format 'yyyy/MM/dd HH:mm:ss')
	[PSCustomObject]@{
		Time = $Time
		Level = $Level
		Message = $Message
	} | Export-Csv -Path "$logname" -Append -NoTypeInformation
	Write-Host "$Time, $Level, $Message"
}

#Function Pause()
#{
#	if ($psISE)
#	{
#		Write-Host ""
#		$null = Read-Host 'Press Enter Key...'
#	}
#	else
#	{
#		Write-Host ""
#		Write-Host "Press Any Key..."
#		(Get-Host).UI.RawUI.ReadKey("NoEcho,IncludeKeyDown") | Out-Null
#	}
#}

# Check for administrator privileges
Function CheckIsAdmin()
{
	try
	{
		Write-Log -Level DEBUG -Message "CheckIsAdmin Start"
		if (-not(([Security.Principal.WindowsPrincipal] `
		[Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole(`
		[Security.Principal.WindowsBuiltInRole] "Administrator"`
		))) {
			Write-Log -Level WARN -Message "Please execute with administrator privileges."
			$null = $wsobj.popup("Please execute with administrator privileges.", 0, "Error", 16)
			exit
		}
		Write-Log -Level DEBUG -Message "CheckIsAdmin End"
	}
	catch
	{
		Write-Log -Level ERROR -Message "------CheckIsAdmin ERROR------"
		Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
		throw "------CheckIsAdmin ERROR------"
	}
}

# Constant definition
$CURRENT_VERSION='HKLM:SOFTWARE\Microsoft\Windows NT\CurrentVersion'
$WINDOWS_UPDATE='HKLM:SOFTWARE\Policies\Microsoft\Windows\WindowsUpdate'

# Function definition
Function GetPCEnvInfo()
{  
	try
	{
		Write-Log -Level DEBUG -Message "GetPCEnvInfo Start"
		if (Test-Path $CURRENT_VERSION)
		{
			Write-Log -Level INFO -Message ("OSName: " + (Get-ItemProperty $CURRENT_VERSION).ProductName)
			Write-Log -Level INFO -Message ("OSVersion: " + (Get-ItemProperty $CURRENT_VERSION).ReleaseId)
			Write-Log -Level INFO -Message ("OSBuild: " + (Get-ItemProperty $CURRENT_VERSION).CurrentBuild+"."+(Get-ItemProperty $CURRENT_VERSION).UBR)
			Write-Log -Level INFO -Message ("VolumeInfo: " + (Get-Volume | Format-Table | Out-String))
		}
		else
		{
			Write-Log -Level INFO -Message "$CURRENT_VERSION is not regist."
		}
		Write-Log -Level DEBUG -Message "GetPCEnvInfo End"
	}
	catch
	{
		Write-Log -Level ERROR -Message "------GetPCEnvInfo ERROR------"
		Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
		throw "------GetPCEnvInfo ERROR------"
	}
}

Function GetRegSettings()
{
	try
	{
		Write-Log -Level DEBUG -Message "GetRegSettings Start"
		if (Test-Path $WINDOWS_UPDATE)
		{
			# Get OS target version
			Write-Log -Level INFO -Message ("ProductVersion: " + (Get-ItemProperty $WINDOWS_UPDATE).ProductVersion)
			Write-Log -Level INFO -Message ("TargetReleaseVersion: " + (Get-ItemProperty $WINDOWS_UPDATE).TargetReleaseVersion)
			Write-Log -Level INFO -Message ("TargetReleaseVersionInfo: " + (Get-ItemProperty $WINDOWS_UPDATE).TargetReleaseVersionInfo)
			
			# Get Defer quality updates
			Write-Log -Level INFO -Message ("DeferQualityUpdates: " + (Get-ItemProperty $WINDOWS_UPDATE).DeferQualityUpdates)
			Write-Log -Level INFO -Message ("DeferQualityUpdatesPeriodInDays: " + (Get-ItemProperty $WINDOWS_UPDATE).DeferQualityUpdatesPeriodInDays)
			Write-Log -Level INFO -Message ("PauseQualityUpdatesStartTime: " + (Get-ItemProperty $WINDOWS_UPDATE).PauseQualityUpdatesStartTime)
		} 
		else
		{
			Write-Log -Level INFO -Message "$WINDOWS_UPDATE is not regist."
		}
		Write-Log -Level DEBUG -Message "GetRegSettings End"
	}
	catch
	{
		Write-Log -Level ERROR -Message "------GetRegSettings ERROR------"
		Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
		throw "------GetRegSettings ERROR------"
	}
}
Function SetRegSettings()
{
	try
	{
		Write-Log -Level DEBUG -Message "SetRegSettings Start"
		if (!(Test-Path $WINDOWS_UPDATE))
		{
			New-Item $WINDOWS_UPDATE
		}
		
		# Set OS target version
		Set-ItemProperty $WINDOWS_UPDATE -name ProductVersion -value 'Windows 10'
		Set-ItemProperty $WINDOWS_UPDATE -name TargetReleaseVersion -value 1
		Set-ItemProperty $WINDOWS_UPDATE -name TargetReleaseVersionInfo -value $argTargetOSVersion
		
		try
		{
			# Set Defer quality updates
			Set-ItemProperty $WINDOWS_UPDATE -name DeferQualityUpdates -value 1
			Set-ItemProperty $WINDOWS_UPDATE -name DeferQualityUpdatesPeriodInDays -value 30
			Set-ItemProperty $WINDOWS_UPDATE -name PauseQualityUpdatesStartTime -value ''
		}
		catch
		{
			# Failure to set is fine
			Write-Log -Level WARN -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
			Write-Log -Level WARN -Message "------Failure to set DeferQualityUpdates is fine.------"
		}
				
		GetRegSettings
		
		Write-Log -Level DEBUG -Message "SetRegSettings End"
	}
	catch
	{
		Write-Log -Level ERROR -Message "------SetRegSettings ERROR------"
		Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
		throw "------SetRegSettings ERROR------"
	}
}
Function RemoveRegSettings()
{
	try
	{
		Write-Log -Level DEBUG -Message "RemoveRegSettings Start"
		if (!(Test-Path $WINDOWS_UPDATE))
		{
			New-Item $WINDOWS_UPDATE
		}
		
        # Set OS target version
		Set-ItemProperty $WINDOWS_UPDATE -name ProductVersion -value ''
		Set-ItemProperty $WINDOWS_UPDATE -name TargetReleaseVersion -value 0
		Set-ItemProperty $WINDOWS_UPDATE -name TargetReleaseVersionInfo -value ''
		
		try
		{
			# Set Defer quality updates
			Set-ItemProperty $WINDOWS_UPDATE -name DeferQualityUpdates -value 0
			Set-ItemProperty $WINDOWS_UPDATE -name DeferQualityUpdatesPeriodInDays -value 0
			Set-ItemProperty $WINDOWS_UPDATE -name PauseQualityUpdatesStartTime -value ''
		}
		catch
		{
			# Failure to set is fine
			Write-Log -Level WARN -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
			Write-Log -Level WARN -Message "------Failure to set DeferQualityUpdates is fine.------"
		}
				
		GetRegSettings
		
		Write-Log -Level DEBUG -Message "RemoveRegSettings End"
	}
	catch
	{
		Write-Log -Level ERROR -Message "------RemoveRegSettings ERROR------"
		Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
		throw "------RemoveRegSettings ERROR------"
	}
}


# PowerShell's trace start
#Start-Transcript -Path $logname

try 
{
    $ResultMessage = ""
	# Main Logic
	CheckIsAdmin
	Write-Log -Level INFO -Message "------Get PC environment information------"
	GetPCEnvInfo
	Write-Log -Level INFO -Message "------Get the configuration Before the change------"
	GetRegSettings
	if($argTargetOSVersion -eq 0)
	{
		Write-Log -Level INFO -Message "------Remove the configuration------"
		RemoveRegSettings
		$ResultMessage = "Remove TargeFeatureUpdateVersion is SUCCESS."
	}
	else
	{
		Write-Log -Level INFO -Message "------Set the new configuration------"
		SetRegSettings
		$ResultMessage = "Set TargeFeatureUpdateVersion is SUCCESS."
	}
	Write-Log -Level INFO -Message "------Get the configuration After the change------"
	GetRegSettings
	Write-Log -Level INFO -Message "------function is success------"
	Write-Host ""
	Write-Log -Level INFO -Message "$ResultMessage"
	
	$null = $wsobj.popup("$ResultMessage", 0, "Success", 64)
}
catch
{
	Write-Log -Level ERROR -Message "------FUNCTION IS FAILED------"
	Write-Host ""
	Write-Log -Level ERROR -Message ($PSItem.Exception.Message + $PSItem.ScriptStackTrace)
	$null = $wsobj.popup("------FUNCTION IS FAILED------", 0, "Error", 16)
	#pause
} 
finally
{
	# PowerShell's trace finish
	#Stop-Transcript
	
	exit
}
